from __future__ import print_function

import onnx
import onnx.helper as oh
from onnx import checker
from get_engine import get_engine
import numpy as np
import tensorrt as trt
import pycuda.driver as cuda
import pycuda.autoinit
import sys
import os
import common

AXIS = 1
ONNX_FILE = "softmax_test.onnx"

# Generate ONNX model
def generate_model():
    in_tensor = [
        oh.make_tensor_value_info("Input", onnx.TensorProto.FLOAT, [1, 1, 3, 5, 5])
    ]
    out_tensor = [
        oh.make_tensor_value_info("Output", onnx.TensorProto.FLOAT, [1, 1, 3, 5, 5])
    ]
    nodes = []
    nodes.append(oh.make_node("Softmax", axis=AXIS, inputs=["Input"], outputs=["Output"]))
    graph = oh.make_graph(nodes, "Test Graph", in_tensor, out_tensor)
    checker.check_graph(graph)
    model = oh.make_model(graph, producer_name="MACNICA", producer_version="0.1")
    checker.check_model(model)
    with open(ONNX_FILE, "wb") as f:
        f.write(model.SerializeToString())

# Main function
def test_model():

    # Load the model into TensorRT
    with get_engine(ONNX_FILE) as engine, \
        engine.create_execution_context() as context:

        # Allocate buffer memory for TensorRT
        inputs, outputs, bindings, stream = common.allocate_buffers(engine)
        
        # Prepare input data
        values = np.arange(3*5*5, dtype=np.float32).reshape((1,1,3,5,5)) / 100
        print('Input:')
        print(values)
        
        # Inference
        inputs[0].host = values
        trt_outputs = common.do_inference(context, bindings=bindings, \
            inputs=inputs, outputs=outputs, stream=stream)
            
        print('Output:')
        print(trt_outputs[0])
        
# Main function
def main():
    generate_model()
    test_model()
  
if __name__ == "__main__":
    main()
